
local QuestAnnounce = LibStub("AceAddon-3.0"):NewAddon("QuestAnnounce", "AceEvent-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("QuestAnnounce")

--[[ The defaults a user without a profile will get. ]]--
local defaults = {
	profile={
		debug = false,
		enable = true,
		every = 1,
		announce = {
			say = false,
			party = true,
			guild = false,
			officer = false,
			whisper = false,
			whisperWho = nil
		}
	}
}

--[[ Quest Announce Initialize ]]--
function QuestAnnounce:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("QuestAnnounceDB", defaults, true)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileReset")
	self.db.RegisterCallback(self, "OnNewProfile", "OnNewProfile")
	
	self:SetupOptions()
end

function QuestAnnounce:OnEnable()
	--[[ We're looking at the UI_INFO_MESSAGE for quest messages ]]--
	self:RegisterEvent("UI_INFO_MESSAGE")
	
	self:SendDebugMsg("Addon Enabled :: "..tostring(QuestAnnounce.db.profile.enable))
end

--[[ Event handler ]]--
function QuestAnnounce:UI_INFO_MESSAGE(event, ...)
	local msg = ...
	if (msg ~= nil) then
		if (QuestAnnounce.db.profile.enable) then
			local questText = gsub(msg, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$", "%1", 1)
			
			if (questText ~= msg) then
				local ii, jj, strItemName, iNumItems, iNumNeeded = string.find(msg, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$")
				local stillNeeded = iNumNeeded - iNumItems
                
				QuestAnnounce:SendDebugMsg("Item Name: "..strItemName.." :: Num Items: "..iNumItems.." :: Num Needed: "..iNumNeeded.." :: Still Need: "..stillNeeded)

				if(stillNeeded == 0 and QuestAnnounce.db.profile.every == 0) then
					QuestAnnounce:SendMsg(L["Completed: "]..msg)
				elseif(QuestAnnounce.db.profile.every > 0) then
					local every = math.fmod(iNumItems, QuestAnnounce.db.profile.every)
					QuestAnnounce:SendDebugMsg("Every fMod: "..every)
				
					if(every == 0 and stillNeeded > 0) then
						QuestAnnounce:SendMsg(L["Progress: "]..msg)
					elseif(stillNeeded == 0) then
						QuestAnnounce:SendMsg(L["Completed: "]..msg)
					end
				end
			end
		end
	end
end

function QuestAnnounce:OnProfileChanged(event, db)
 	self.db.profile = db.profile
end

function QuestAnnounce:OnProfileReset(event, db)
	for k, v in pairs(defaults) do
		db.profile[k] = v
	end
	self.db.profile = db.profile
end

function QuestAnnounce:OnNewProfile(event, db)
	for k, v in pairs(defaults) do
		db.profile[k] = v
	end
end

--[[ Sends a debugging message if debug is enabled and we have a message to send ]]--
function QuestAnnounce:SendDebugMsg(msg)
	if(msg ~= nil and self.db.profile.debug) then
		QuestAnnounce:Print("DEBUG :: "..msg)
	end
end

--[[ Sends a chat message to the selected chat channels if we have a message to send; will also send a debugging message if debug is enabled ]]--
function QuestAnnounce:SendMsg(msg)	
	if (msg ~= nil and self.db.profile.enable) then
		if(self.db.profile.announce.say) then
			SendChatMessage(msg, "SAY")
			QuestAnnounce:SendDebugMsg("QuestAnnounce:SendMsg :: Say")
		end
		
		if(self.db.profile.announce.party and GetNumPartyMembers() > 0) then
			SendChatMessage(msg, "PARTY")
			QuestAnnounce:SendDebugMsg("QuestAnnounce:SendMsg :: Party")
		end
		
		if(IsInGuild()) then
			if(self.db.profile.announce.guild) then
				SendChatMessage(msg, "GUILD")
				QuestAnnounce:SendDebugMsg("QuestAnnounce:SendMsg :: Guild")
			end
		
			if(self.db.profile.announce.officer) then
				SendChatMessage(msg, "OFFICER")
				QuestAnnounce:SendDebugMsg("QuestAnnounce:SendMsg :: Officer")
			end
		end
				
		if(self.db.profile.announce.whisper) then
			local who = self.db.profile.announce.whisperWho
			if(who ~= nil and who ~= "") then
				SendChatMessage(msg, "WHISPER", nil, who)
				QuestAnnounce:SendDebugMsg("QuestAnnounce:SendMsg :: Whisper")
			end
		end
	end
	QuestAnnounce:SendDebugMsg("QuestAnnounce:SendMsg - "..msg)
end